package com.sun.jmx.snmp.agent;

import com.sun.jmx.snmp.SnmpOid;
import com.sun.jmx.snmp.SnmpVarBind;
import java.util.Vector;
import com.sun.jmx.snmp.SnmpStatusException;
import java.util.Enumeration;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanRegistrationException;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import java.io.Serializable;

public abstract class SnmpMib extends SnmpMibAgent implements Serializable
{
    protected SnmpMibOid root;
    private transient long[] rootOid;
    
    public SnmpMib() {
        this.rootOid = null;
        this.root = new SnmpMibOid();
    }
    
    protected String getGroupOid(String s, String s2) {
        return s2;
    }
    
    protected ObjectName getGroupObjectName(String s, String s2, String name) throws MalformedObjectNameException {
        return new ObjectName(name);
    }
    
    protected void registerGroupNode(String s, String s2, ObjectName objectName, SnmpMibNode snmpMibNode, Object o, MBeanServer mBeanServer) throws NotCompliantMBeanException, MBeanRegistrationException, InstanceAlreadyExistsException, IllegalAccessException {
        this.root.registerNode(s2, snmpMibNode);
        if (mBeanServer != null && objectName != null && o != null) {
            mBeanServer.registerMBean(o, objectName);
        }
    }
    
    public abstract void registerTableMeta(String p0, SnmpMibTable p1);
    
    public abstract SnmpMibTable getRegisteredTableMeta(String p0);
    
    @Override
    public void get(SnmpMibRequest snmpMibRequest) throws SnmpStatusException {
        SnmpRequestTree handlers = this.getHandlers(snmpMibRequest, false, false, 160);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "get", "Processing handlers for GET... ");
        }
        Enumeration<SnmpRequestTree.Handler> handlers2 = handlers.getHandlers();
        while (handlers2.hasMoreElements()) {
            SnmpRequestTree.Handler handler = (SnmpRequestTree.Handler)handlers2.nextElement();
            SnmpMibNode metaNode = handlers.getMetaNode(handler);
            int oidDepth = handlers.getOidDepth(handler);
            Enumeration<SnmpMibSubRequest> subRequests = handlers.getSubRequests(handler);
            while (subRequests.hasMoreElements()) {
                metaNode.get((SnmpMibSubRequest)subRequests.nextElement(), oidDepth);
            }
        }
    }
    
    @Override
    public void set(SnmpMibRequest snmpMibRequest) throws SnmpStatusException {
        SnmpRequestTree snmpRequestTree = null;
        if ((snmpMibRequest instanceof SnmpMibRequestImpl)) {
            snmpRequestTree = ((SnmpMibRequestImpl)snmpMibRequest).getRequestTree();
        }
        if (snmpRequestTree == null) {
            snmpRequestTree = this.getHandlers(snmpMibRequest, false, true, 163);
        }
        snmpRequestTree.switchCreationFlag(false);
        snmpRequestTree.setPduType(163);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "set", "Processing handlers for SET... ");
        }
        Enumeration<SnmpRequestTree.Handler> handlers = snmpRequestTree.getHandlers();
        while (handlers.hasMoreElements()) {
            SnmpRequestTree.Handler handler = (SnmpRequestTree.Handler)handlers.nextElement();
            SnmpMibNode metaNode = snmpRequestTree.getMetaNode(handler);
            int oidDepth = snmpRequestTree.getOidDepth(handler);
            Enumeration<SnmpMibSubRequest> subRequests = snmpRequestTree.getSubRequests(handler);
            while (subRequests.hasMoreElements()) {
                metaNode.set((SnmpMibSubRequest)subRequests.nextElement(), oidDepth);
            }
        }
    }
    
    @Override
    public void check(SnmpMibRequest snmpMibRequest) throws SnmpStatusException {
        SnmpRequestTree handlers = this.getHandlers(snmpMibRequest, true, true, 253);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "check", "Processing handlers for CHECK... ");
        }
        Enumeration<SnmpRequestTree.Handler> handlers2 = handlers.getHandlers();
        while (handlers2.hasMoreElements()) {
            SnmpRequestTree.Handler handler = (SnmpRequestTree.Handler)handlers2.nextElement();
            SnmpMibNode metaNode = handlers.getMetaNode(handler);
            int oidDepth = handlers.getOidDepth(handler);
            Enumeration<SnmpMibSubRequest> subRequests = handlers.getSubRequests(handler);
            while (subRequests.hasMoreElements()) {
                metaNode.check((SnmpMibSubRequest)subRequests.nextElement(), oidDepth);
            }
        }
        if ((snmpMibRequest instanceof SnmpMibRequestImpl)) {
            ((SnmpMibRequestImpl)snmpMibRequest).setRequestTree(handlers);
        }
    }
    
    @Override
    public void getNext(SnmpMibRequest snmpMibRequest) throws SnmpStatusException {
        SnmpRequestTree getNextHandlers = this.getGetNextHandlers(snmpMibRequest);
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getNext", "Processing handlers for GET-NEXT... ");
        }
        Enumeration<SnmpRequestTree.Handler> handlers = getNextHandlers.getHandlers();
        while (handlers.hasMoreElements()) {
            SnmpRequestTree.Handler handler = (SnmpRequestTree.Handler)handlers.nextElement();
            SnmpMibNode metaNode = getNextHandlers.getMetaNode(handler);
            int oidDepth = getNextHandlers.getOidDepth(handler);
            Enumeration<SnmpMibSubRequest> subRequests = getNextHandlers.getSubRequests(handler);
            while (subRequests.hasMoreElements()) {
                metaNode.get((SnmpMibSubRequest)subRequests.nextElement(), oidDepth);
            }
        }
    }
    
    @Override
    public void getBulk(SnmpMibRequest snmpMibRequest, int n, int n2) throws SnmpStatusException {
        this.getBulkWithGetNext(snmpMibRequest, n, n2);
    }
    
    @Override
    public long[] getRootOid() {
        if (this.rootOid == null) {
            Vector vector = new Vector(10);
            this.root.getRootOid(vector);
            this.rootOid = new long[vector.size()];
            int n = 0;
            Enumeration<Object> elements = vector.elements();
            while (elements.hasMoreElements()) {
                this.rootOid[n++] = (long)(Integer)elements.nextElement();
            }
        }
        return (long[])this.rootOid.clone();
    }
    
    private SnmpRequestTree getHandlers(SnmpMibRequest snmpMibRequest, boolean b, boolean b2, int n) throws SnmpStatusException {
        SnmpRequestTree snmpRequestTree = new SnmpRequestTree(snmpMibRequest, b, n);
        int n2 = 0;
        int version = snmpMibRequest.getVersion();
        for (Enumeration elements = snmpMibRequest.getElements(); elements.hasMoreElements(); ++n2) {
            SnmpVarBind snmpVarBind = (SnmpVarBind)elements.nextElement();
            try {
                this.root.findHandlingNode(snmpVarBind, snmpVarBind.oid.longValue(false), 0, snmpRequestTree);
            }
            catch (SnmpStatusException ex) {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "Couldn't find a handling node for " + snmpVarBind.oid.toString());
                }
                if (version == 0) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tV1: Throwing exception");
                    }
                    SnmpStatusException o2 = new SnmpStatusException(ex, n2 + 1);
                    o2.initCause(ex);
                    throw o2;
                }
                else if (n == 253 || n == 163) {
                    int mapSetException = SnmpRequestTree.mapSetException(ex.getStatus(), version);
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tSET: Throwing exception");
                    }
                    SnmpStatusException ex3 = new SnmpStatusException(mapSetException, n2 + 1);
                    ex3.initCause(ex);
                    throw ex3;
                }
                else if (b2) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tATOMIC: Throwing exception");
                    }
                    SnmpStatusException o = new SnmpStatusException(ex, n2 + 1);
                    o.initCause(ex);
                    throw o;
                }
                else {
                    int mapGetException = SnmpRequestTree.mapGetException(ex.getStatus(), version);
                    if (mapGetException == 224) {
                        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tGET: Registering noSuchInstance");
                        }
                        snmpVarBind.value = SnmpVarBind.noSuchInstance;
                    }
                    else if (mapGetException == 225) {
                        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tGET: Registering noSuchObject");
                        }
                        snmpVarBind.value = SnmpVarBind.noSuchObject;
                    }
                    else {
                        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getHandlers", "\tGET: Registering global error: " + mapGetException);
                        }
                        SnmpStatusException ex2 = new SnmpStatusException(mapGetException, n2 + 1);
                        ex2.initCause(ex);
                        throw ex2;
                    }
                }
            }
        }
        return snmpRequestTree;
    }
    
    private SnmpRequestTree getGetNextHandlers(SnmpMibRequest obj) throws SnmpStatusException {
        SnmpRequestTree snmpRequestTree = new SnmpRequestTree(obj, false, 161);
        snmpRequestTree.setGetNextFlag();
        if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
            JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getGetNextHandlers", "Received MIB request : " + obj);
        }
        AcmChecker acmChecker = new AcmChecker(obj);
        int n = 0;
        int version = obj.getVersion();
        for (Enumeration elements = obj.getElements(); elements.hasMoreElements(); ++n) {
            SnmpVarBind snmpVarBind = (SnmpVarBind)elements.nextElement();
            try {
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getGetNextHandlers", " Next OID of : " + snmpVarBind.oid);
                }
                SnmpOid o = new SnmpOid(this.root.findNextHandlingNode(snmpVarBind, snmpVarBind.oid.longValue(false), 0, 0, snmpRequestTree, acmChecker));
                if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getGetNextHandlers", " is : " + o);
                }
                snmpVarBind.oid = o;
            }
            catch (SnmpStatusException ex) {
                if (version == 0) {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getGetNextHandlers", "\tThrowing exception " + ex.toString());
                    }
                    throw new SnmpStatusException(ex, n + 1);
                }
                else {
                    if (JmxProperties.SNMP_ADAPTOR_LOGGER.isLoggable(Level.FINEST)) {
                        JmxProperties.SNMP_ADAPTOR_LOGGER.logp(Level.FINEST, SnmpMib.class.getName(), "getGetNextHandlers", "Exception : " + ex.getStatus());
                    }
                    snmpVarBind.setSnmpValue(SnmpVarBind.endOfMibView);
                }
            }
        }
        return snmpRequestTree;
    }
}
